@@ -51,6 +51,21 @@ pub trait Matrix2x2<T: NumEx, V2: Vector2<T>>: Matrix<T> {
         [self.x_axis().into_array(), self.y_axis().into_array()]
     }
 
+    #[inline(always)]
+    fn from_cols_slice(m: &[T]) -> Self {
+        Self::new(m[0], m[1], m[2], m[3])
+    }
+
+    #[inline(always)]
+    fn write_cols_to_slice(&self, slice: &mut [T]) {
+        let x_axis = self.x_axis();
+        let y_axis = self.y_axis();
+        slice[0] = x_axis.x();
+        slice[1] = x_axis.y();
+        slice[2] = y_axis.x();
+        slice[3] = y_axis.y();
+    }
+
     #[rustfmt::skip]
     #[inline(always)]
     fn from_diagonal(diagonal: XY<T>) -> Self {
@@ -128,6 +143,11 @@ pub trait FloatMatrix2x2<T: FloatEx, V2: FloatVector2<T>>: Matrix2x2<T, V2> {
             && self.y_axis().abs_diff_eq(*other.y_axis(), max_abs_diff)
     }
 
+    #[inline]
+    fn neg_matrix(&self) -> Self {
+        Self::from_cols(self.x_axis().neg(), self.y_axis().neg())
+    }
+
     #[inline]
     fn from_scale_angle(scale: V2, angle: T) -> Self {
         let (sin, cos) = angle.sin_cos();
@@ -206,6 +226,31 @@ pub trait Matrix3x3<T: NumEx, V3: Vector3<T>>: Matrix<T> {
         ]
     }
 
+    #[inline(always)]
+    fn from_cols_slice(m: &[T]) -> Self {
+        Self::from_cols(
+            V3::new(m[0], m[1], m[2]),
+            V3::new(m[3], m[4], m[5]),
+            V3::new(m[6], m[7], m[8]),
+        )
+    }
+
+    #[inline(always)]
+    fn write_cols_to_slice(&self, slice: &mut [T]) {
+        let x_axis = self.x_axis();
+        let y_axis = self.y_axis();
+        let z_axis = self.z_axis();
+        slice[0] = x_axis.x();
+        slice[1] = x_axis.y();
+        slice[2] = x_axis.z();
+        slice[3] = y_axis.x();
+        slice[4] = y_axis.y();
+        slice[5] = y_axis.z();
+        slice[6] = z_axis.x();
+        slice[7] = z_axis.y();
+        slice[8] = z_axis.z();
+    }
+
     #[rustfmt::skip]
     #[inline(always)]
     fn from_diagonal(diagonal: XYZ<T>) -> Self {
@@ -305,6 +350,15 @@ pub trait FloatMatrix3x3<T: FloatEx, V3: FloatVector3<T>>: Matrix3x3<T, V3> {
             && self.z_axis().abs_diff_eq(*other.z_axis(), max_abs_diff)
     }
 
+    #[inline]
+    fn neg_matrix(&self) -> Self {
+        Self::from_cols(
+            self.x_axis().neg(),
+            self.y_axis().neg(),
+            self.z_axis().neg(),
+        )
+    }
+
     #[inline]
     fn from_angle(angle: T) -> Self {
         let (sin, cos) = angle.sin_cos();
@@ -480,6 +534,43 @@ pub trait Matrix4x4<T: NumEx, V4: Vector4<T>>: Matrix<T> {
         ]
     }
 
+    #[rustfmt::skip]
+    #[inline(always)]
+    fn from_cols_slice(m: &[T]) -> Self {
+        Self::from_cols(
+            V4::new( m[0],  m[1],  m[2],  m[3]),
+            V4::new( m[4],  m[5],  m[6],  m[7]),
+            V4::new( m[8],  m[9], m[10], m[11]),
+            V4::new(m[12], m[13], m[14], m[15]))
+    }
+
+    #[inline(always)]
+    fn write_cols_to_slice(&self, slice: &mut [T]) {
+        let x_axis = self.x_axis();
+        let y_axis = self.y_axis();
+        let z_axis = self.z_axis();
+        let w_axis = self.w_axis();
+        slice[0] = x_axis.x();
+        slice[1] = x_axis.y();
+        slice[2] = x_axis.z();
+        slice[3] = x_axis.w();
+
+        slice[4] = y_axis.x();
+        slice[5] = y_axis.y();
+        slice[6] = y_axis.z();
+        slice[7] = y_axis.w();
+
+        slice[8] = z_axis.x();
+        slice[9] = z_axis.y();
+        slice[10] = z_axis.z();
+        slice[11] = z_axis.w();
+
+        slice[12] = w_axis.x();
+        slice[13] = w_axis.y();
+        slice[14] = w_axis.z();
+        slice[15] = w_axis.w();
+    }
+
     #[inline(always)]
     fn from_diagonal(diagonal: XYZW<T>) -> Self {
         Self::from_cols(
@@ -548,7 +639,7 @@ pub trait Matrix4x4<T: NumEx, V4: Vector4<T>>: Matrix<T> {
     }
 
     #[inline]
-    fn mul_vector(&self, other: &V4) -> V4 {
+    fn mul_vector(&self, other: V4) -> V4 {
         let mut res = self.x_axis().mul(other.splat_x());
         res = self.y_axis().mul_add(other.splat_y(), res);
         res = self.z_axis().mul_add(other.splat_z(), res);
@@ -559,10 +650,10 @@ pub trait Matrix4x4<T: NumEx, V4: Vector4<T>>: Matrix<T> {
     #[inline]
     fn mul_matrix(&self, other: &Self) -> Self {
         Self::from_cols(
-            self.mul_vector(other.x_axis()),
-            self.mul_vector(other.y_axis()),
-            self.mul_vector(other.z_axis()),
-            self.mul_vector(other.w_axis()),
+            self.mul_vector(*other.x_axis()),
+            self.mul_vector(*other.y_axis()),
+            self.mul_vector(*other.z_axis()),
+            self.mul_vector(*other.w_axis()),
         )
     }
 
@@ -616,6 +707,16 @@ pub trait FloatMatrix4x4<T: FloatEx, V4: FloatVector4<T> + Quaternion<T>>:
             && self.w_axis().abs_diff_eq(*other.w_axis(), max_abs_diff)
     }
 
+    #[inline]
+    fn neg_matrix(&self) -> Self {
+        Self::from_cols(
+            self.x_axis().neg(),
+            self.y_axis().neg(),
+            self.z_axis().neg(),
+            self.w_axis().neg(),
+        )
+    }
+
     #[inline]
     fn quaternion_to_axes(rotation: V4) -> (V4, V4, V4) {
         glam_assert!(rotation.is_normalized());
@@ -641,7 +742,6 @@ pub trait FloatMatrix4x4<T: FloatEx, V4: FloatVector4<T> + Quaternion<T>>:
 
     #[inline]
     fn from_quaternion(rotation: V4) -> Self {
-        glam_assert!(rotation.is_normalized());
         let (x_axis, y_axis, z_axis) = Self::quaternion_to_axes(rotation);
         Self::from_cols(x_axis, y_axis, z_axis, V4::W)
     }
@@ -673,7 +773,6 @@ pub trait FloatMatrix4x4<T: FloatEx, V4: FloatVector4<T> + Quaternion<T>>:
 
     #[inline]
     fn from_scale_quaternion_translation(scale: XYZ<T>, rotation: V4, translation: XYZ<T>) -> Self {
-        glam_assert!(rotation.is_normalized());
         let (x_axis, y_axis, z_axis) = Self::quaternion_to_axes(rotation);
         Self::from_cols(
             x_axis.mul_scalar(scale.x),
@@ -685,7 +784,6 @@ pub trait FloatMatrix4x4<T: FloatEx, V4: FloatVector4<T> + Quaternion<T>>:
 
     #[inline]
     fn from_quaternion_translation(rotation: V4, translation: XYZ<T>) -> Self {
-        glam_assert!(rotation.is_normalized());
         let (x_axis, y_axis, z_axis) = Self::quaternion_to_axes(rotation);
         Self::from_cols(x_axis, y_axis, z_axis, V4::from_xyz(translation, T::ONE))
     }
