@@ -3,14 +3,12 @@ mod support;
 
 macro_rules! impl_vec2_tests {
     ($t:ty, $const_new:ident, $new:ident, $vec2:ident, $vec3:ident, $mask:ident) => {
-        #[test]
-        fn test_const() {
+        glam_test!(test_const, {
             const V: $vec2 = $const_new!([1 as $t, 2 as $t]);
             assert_eq!($vec2::new(1 as $t, 2 as $t), V);
-        }
+        });
 
-        #[test]
-        fn test_new() {
+        glam_test!(test_new, {
             let v = $new(1 as $t, 2 as $t);
 
             assert_eq!(v.x, 1 as $t);
@@ -30,10 +28,9 @@ macro_rules! impl_vec2_tests {
 
             assert_eq!($vec2::new(1 as $t, 0 as $t), $vec2::X);
             assert_eq!($vec2::new(0 as $t, 1 as $t), $vec2::Y);
-        }
+        });
 
-        #[test]
-        fn test_fmt() {
+        glam_test!(test_fmt, {
             let a = $vec2::new(1 as $t, 2 as $t);
             assert_eq!(
                 format!("{:?}", a),
@@ -41,23 +38,20 @@ macro_rules! impl_vec2_tests {
             );
             // assert_eq!(format!("{:#?}", a), "$vec2(\n    1.0,\n    2.0\n)");
             assert_eq!(format!("{}", a), "[1, 2]");
-        }
+        });
 
-        #[test]
-        fn test_zero() {
+        glam_test!(test_zero, {
             let v = $vec2::ZERO;
             assert_eq!($new(0 as $t, 0 as $t), v);
             assert_eq!(v, $vec2::default());
-        }
+        });
 
-        #[test]
-        fn test_splat() {
+        glam_test!(test_splat, {
             let v = $vec2::splat(1 as $t);
             assert_eq!($vec2::ONE, v);
-        }
+        });
 
-        #[test]
-        fn test_accessors() {
+        glam_test!(test_accessors, {
             let mut a = $vec2::ZERO;
             a.x = 1 as $t;
             a.y = 2 as $t;
@@ -71,18 +65,16 @@ macro_rules! impl_vec2_tests {
             assert_eq!(1 as $t, a[0]);
             assert_eq!(2 as $t, a[1]);
             assert_eq!($vec2::new(1 as $t, 2 as $t), a);
-        }
+        });
 
-        #[test]
-        fn test_dot_unsigned() {
+        glam_test!(test_dot_unsigned, {
             let x = $new(1 as $t, 0 as $t);
             let y = $new(0 as $t, 1 as $t);
             assert_eq!(1 as $t, x.dot(x));
             assert_eq!(0 as $t, x.dot(y));
-        }
+        });
 
-        #[test]
-        fn test_ops() {
+        glam_test!(test_ops, {
             let a = $new(2 as $t, 4 as $t);
             assert_eq!($new(4 as $t, 8 as $t), (a + a));
             assert_eq!($new(0 as $t, 0 as $t), (a - a));
@@ -92,10 +84,15 @@ macro_rules! impl_vec2_tests {
             assert_eq!($new(1 as $t, 1 as $t), (a / a));
             assert_eq!($new(1 as $t, 2 as $t), (a / 2 as $t));
             assert_eq!($new(2 as $t, 1 as $t), (4 as $t / a));
-        }
+            assert_eq!($new(0 as $t, 0 as $t), a % a);
+            assert_eq!($new(0 as $t, 1 as $t), a % (a - 1 as $t));
+            assert_eq!($new(0 as $t, 0 as $t), a % 1 as $t);
+            assert_eq!($new(2 as $t, 1 as $t), a % 3 as $t);
+            assert_eq!($new(1 as $t, 1 as $t), 17 as $t % a);
+            assert_eq!($new(2 as $t, 4 as $t), a % 8 as $t);
+        });
 
-        #[test]
-        fn test_assign_ops() {
+        glam_test!(test_assign_ops, {
             let a = $new(1 as $t, 2 as $t);
             let mut b = a;
             b += a;
@@ -110,20 +107,22 @@ macro_rules! impl_vec2_tests {
             assert_eq!($new(2 as $t, 4 as $t), b);
             b /= 2 as $t;
             assert_eq!($new(1 as $t, 2 as $t), b);
-        }
+            b %= (b + 1 as $t);
+            assert_eq!($new(1 as $t, 2 as $t), b);
+            b %= b;
+            assert_eq!($new(0 as $t, 0 as $t), b);
+        });
 
-        #[test]
-        fn test_min_max() {
+        glam_test!(test_min_max, {
             let a = $new(0 as $t, 2 as $t);
             let b = $new(1 as $t, 1 as $t);
             assert_eq!($new(0 as $t, 1 as $t), a.min(b));
             assert_eq!($new(0 as $t, 1 as $t), b.min(a));
             assert_eq!($new(1 as $t, 2 as $t), a.max(b));
             assert_eq!($new(1 as $t, 2 as $t), b.max(a));
-        }
+        });
 
-        #[test]
-        fn test_clamp() {
+        glam_test!(test_clamp, {
             fn vec(x: i32, y: i32) -> $vec2 {
                 $vec2::new(x as $t, y as $t)
             }
@@ -137,17 +136,15 @@ macro_rules! impl_vec2_tests {
             assert_eq!(vec(9, 9).clamp(min, max), vec(6, 8));
 
             should_glam_assert!({ $vec2::clamp($vec2::ZERO, $vec2::ONE, $vec2::ZERO) });
-        }
+        });
 
-        #[test]
-        fn test_hmin_hmax() {
+        glam_test!(test_hmin_hmax, {
             let a = $new(1 as $t, 2 as $t);
             assert_eq!(1 as $t, a.min_element());
             assert_eq!(2 as $t, a.max_element());
-        }
+        });
 
-        #[test]
-        fn test_eq() {
+        glam_test!(test_eq, {
             let a = $new(1 as $t, 1 as $t);
             let b = $new(1 as $t, 2 as $t);
             assert!(a.cmpeq(a).all());
@@ -155,10 +152,9 @@ macro_rules! impl_vec2_tests {
             assert!(a.cmpne(b).any());
             assert!(b.cmpne(a).any());
             assert!(b.cmpeq(a).any());
-        }
+        });
 
-        #[test]
-        fn test_cmp() {
+        glam_test!(test_cmp, {
             assert!(!$mask::default().any());
             assert!(!$mask::default().all());
             assert_eq!($mask::default().bitmask(), 0x0);
@@ -181,17 +177,15 @@ macro_rules! impl_vec2_tests {
             assert!(c.cmple(c).all());
             assert!(c.cmpge(c).all());
             assert!(a == a);
-        }
+        });
 
-        #[test]
-        fn test_extend_truncate() {
+        glam_test!(test_extend_truncate, {
             let a = $new(1 as $t, 2 as $t);
             let b = a.extend(3 as $t);
             assert_eq!($vec3::new(1 as $t, 2 as $t, 3 as $t), b);
-        }
+        });
 
-        #[test]
-        fn test_vec2mask() {
+        glam_test!(test_vec2mask, {
             // make sure the unused 'z' value doesn't break $vec2 behaviour
             let a = $vec3::ZERO;
             let mut b = a.truncate();
@@ -199,7 +193,7 @@ macro_rules! impl_vec2_tests {
             b.y = 1 as $t;
             assert!(!b.cmpeq($vec2::ZERO).any());
             assert!(b.cmpeq($vec2::splat(1 as $t)).all());
-        }
+        });
 
         // #[test]
         // fn test_mask_as_ref() {
@@ -209,40 +203,35 @@ macro_rules! impl_vec2_tests {
         //     assert_eq!($mask::new(true, true).as_ref(), &[!0, !0]);
         // }
 
-        #[test]
-        fn test_mask_from() {
+        glam_test!(test_mask_from, {
             assert_eq!(Into::<[u32; 2]>::into($mask::new(false, false)), [0, 0]);
             assert_eq!(Into::<[u32; 2]>::into($mask::new(true, false)), [!0, 0]);
             assert_eq!(Into::<[u32; 2]>::into($mask::new(false, true)), [0, !0]);
             assert_eq!(Into::<[u32; 2]>::into($mask::new(true, true)), [!0, !0]);
-        }
+        });
 
-        #[test]
-        fn test_mask_bitmask() {
+        glam_test!(test_mask_bitmask, {
             assert_eq!($mask::new(false, false).bitmask(), 0b00);
             assert_eq!($mask::new(true, false).bitmask(), 0b01);
             assert_eq!($mask::new(false, true).bitmask(), 0b10);
             assert_eq!($mask::new(true, true).bitmask(), 0b11);
-        }
+        });
 
-        #[test]
-        fn test_mask_any() {
+        glam_test!(test_mask_any, {
             assert_eq!($mask::new(false, false).any(), false);
             assert_eq!($mask::new(true, false).any(), true);
             assert_eq!($mask::new(false, true).any(), true);
             assert_eq!($mask::new(true, true).any(), true);
-        }
+        });
 
-        #[test]
-        fn test_mask_all() {
+        glam_test!(test_mask_all, {
             assert_eq!($mask::new(false, false).all(), false);
             assert_eq!($mask::new(true, false).all(), false);
             assert_eq!($mask::new(false, true).all(), false);
             assert_eq!($mask::new(true, true).all(), true);
-        }
+        });
 
-        #[test]
-        fn test_mask_select() {
+        glam_test!(test_mask_select, {
             let a = $vec2::new(1 as $t, 2 as $t);
             let b = $vec2::new(3 as $t, 4 as $t);
             assert_eq!(
@@ -261,10 +250,9 @@ macro_rules! impl_vec2_tests {
                 $vec2::select($mask::new(false, false), a, b),
                 $vec2::new(3 as $t, 4 as $t),
             );
-        }
+        });
 
-        #[test]
-        fn test_mask_and() {
+        glam_test!(test_mask_and, {
             assert_eq!(
                 ($mask::new(false, false) & $mask::new(false, false)).bitmask(),
                 0b00,
@@ -285,10 +273,9 @@ macro_rules! impl_vec2_tests {
             let mut mask = $mask::new(true, true);
             mask &= $mask::new(true, false);
             assert_eq!(mask.bitmask(), 0b01);
-        }
+        });
 
-        #[test]
-        fn test_mask_or() {
+        glam_test!(test_mask_or, {
             assert_eq!(
                 ($mask::new(false, false) | $mask::new(false, false)).bitmask(),
                 0b00,
@@ -309,18 +296,16 @@ macro_rules! impl_vec2_tests {
             let mut mask = $mask::new(true, true);
             mask |= $mask::new(true, false);
             assert_eq!(mask.bitmask(), 0b11);
-        }
+        });
 
-        #[test]
-        fn test_mask_not() {
+        glam_test!(test_mask_not, {
             assert_eq!((!$mask::new(false, false)).bitmask(), 0b11);
             assert_eq!((!$mask::new(true, false)).bitmask(), 0b10);
             assert_eq!((!$mask::new(false, true)).bitmask(), 0b01);
             assert_eq!((!$mask::new(true, true)).bitmask(), 0b00);
-        }
+        });
 
-        #[test]
-        fn test_mask_fmt() {
+        glam_test!(test_mask_fmt, {
             let a = $mask::new(true, false);
 
             assert_eq!(
@@ -328,10 +313,9 @@ macro_rules! impl_vec2_tests {
                 format!("{}(0xffffffff, 0x0)", stringify!($mask))
             );
             assert_eq!(format!("{}", a), "[true, false]");
-        }
+        });
 
-        #[test]
-        fn test_mask_eq() {
+        glam_test!(test_mask_eq, {
             let a = $mask::new(true, false);
             let b = $mask::new(true, false);
             let c = $mask::new(false, true);
@@ -340,10 +324,9 @@ macro_rules! impl_vec2_tests {
             assert_eq!(b, a);
             assert_ne!(a, c);
             assert_ne!(b, c);
-        }
+        });
 
-        #[test]
-        fn test_mask_hash() {
+        glam_test!(test_mask_hash, {
             use std::collections::hash_map::DefaultHasher;
             use std::hash::Hash;
             use std::hash::Hasher;
@@ -368,10 +351,9 @@ macro_rules! impl_vec2_tests {
             assert_eq!(a_hashed, b_hashed);
             assert_ne!(a, c);
             assert_ne!(a_hashed, c_hashed);
-        }
+        });
 
-        #[test]
-        fn test_to_from_slice() {
+        glam_test!(test_to_from_slice, {
             let v = $vec2::new(1 as $t, 2 as $t);
             let mut a = [0 as $t, 0 as $t];
             v.write_to_slice(&mut a);
@@ -379,21 +361,17 @@ macro_rules! impl_vec2_tests {
 
             should_panic!({ $vec2::ONE.write_to_slice(&mut [0 as $t]) });
             should_panic!({ $vec2::from_slice(&[0 as $t]) });
-        }
+        });
 
-        #[cfg(feature = "std")]
-        #[test]
-        fn test_sum() {
+        glam_test!(test_sum, {
             let one = $vec2::ONE;
             assert_eq!(vec![one, one].iter().sum::<$vec2>(), one + one);
-        }
+        });
 
-        #[cfg(feature = "std")]
-        #[test]
-        fn test_product() {
+        glam_test!(test_product, {
             let two = $vec2::new(2 as $t, 2 as $t);
             assert_eq!(vec![two, two].iter().product::<$vec2>(), two * two);
-        }
+        });
     };
 }
 
@@ -401,27 +379,24 @@ macro_rules! impl_vec2_signed_tests {
     ($t:ident, $const_new:ident, $new:ident, $vec2:ident, $vec3:ident, $mask:ident) => {
         impl_vec2_tests!($t, $const_new, $new, $vec2, $vec3, $mask);
 
-        #[test]
-        fn test_dot_signed() {
+        glam_test!(test_dot_signed, {
             let x = $new(1 as $t, 0 as $t);
             let y = $new(0 as $t, 1 as $t);
             assert_eq!(1 as $t, x.dot(x));
             assert_eq!(0 as $t, x.dot(y));
             assert_eq!(-1 as $t, x.dot(-x));
-        }
+        });
 
-        #[test]
-        fn test_neg() {
+        glam_test!(test_neg, {
             let a = $new(1 as $t, 2 as $t);
             assert_eq!($new(-1 as $t, -2 as $t), (-a));
-        }
+        });
     };
 }
 
 macro_rules! impl_vec2_eq_hash_tests {
     ($t:ident, $new:ident) => {
-        #[test]
-        fn test_ve2_hash() {
+        glam_test!(test_ve2_hash, {
             use std::collections::hash_map::DefaultHasher;
             use std::hash::Hash;
             use std::hash::Hasher;
@@ -446,7 +421,7 @@ macro_rules! impl_vec2_eq_hash_tests {
             assert_eq!(a_hashed, b_hashed);
             assert_ne!(a, c);
             assert_ne!(a_hashed, c_hashed);
-        }
+        });
     };
 }
 
@@ -459,16 +434,19 @@ macro_rules! impl_vec2_float_tests {
         use core::$t::NAN;
         use core::$t::NEG_INFINITY;
 
-        #[test]
-        fn test_vec2_consts() {
+        glam_test!(test_vec2_consts, {
             assert_eq!($vec2::ZERO, $new(0 as $t, 0 as $t));
             assert_eq!($vec2::ONE, $new(1 as $t, 1 as $t));
             assert_eq!($vec2::X, $new(1 as $t, 0 as $t));
             assert_eq!($vec2::Y, $new(0 as $t, 1 as $t));
-        }
+        });
 
-        #[test]
-        fn test_length() {
+        glam_test!(test_vec2_nan, {
+            assert!($vec2::NAN.is_nan());
+            assert!(!$vec2::NAN.is_finite());
+        });
+
+        glam_test!(test_length, {
             let x = $new(1.0, 0.0);
             let y = $new(0.0, 1.0);
             assert_eq!(4.0, (2.0 * x).length_squared());
@@ -497,10 +475,9 @@ macro_rules! impl_vec2_float_tests {
                 $new(2.0, 3.0).normalize()
             );
             assert_eq!($new(0.5, 0.25), $new(2.0, 4.0).recip());
-        }
+        });
 
-        #[test]
-        fn test_project_reject() {
+        glam_test!(test_project_reject, {
             assert_eq!($new(0.0, 1.0), $new(1.0, 1.0).project_onto($new(0.0, 2.0)));
             assert_eq!($new(1.0, 0.0), $new(1.0, 1.0).reject_from($new(0.0, 2.0)));
             assert_eq!(
@@ -515,9 +492,9 @@ macro_rules! impl_vec2_float_tests {
             should_glam_assert!({ $vec2::ONE.reject_from($vec2::ZERO) });
             should_glam_assert!({ $vec2::ONE.project_onto_normalized($vec2::ONE) });
             should_glam_assert!({ $vec2::ONE.reject_from_normalized($vec2::ONE) });
-        }
-        #[test]
-        fn test_perp() {
+        });
+
+        glam_test!(test_perp, {
             let v1 = $vec2::new(1.0, 2.0);
             let v2 = $vec2::new(1.0, 1.0);
             let v1_perp = $vec2::new(-2.0, 1.0);
@@ -529,10 +506,9 @@ macro_rules! impl_vec2_float_tests {
             assert_eq!(v1.perp().dot(v2), v1.perp_dot(v2));
 
             assert_approx_eq!(v1.perp(), rot90 * v1);
-        }
+        });
 
-        #[test]
-        fn test_sign() {
+        glam_test!(test_sign, {
             assert_eq!($vec2::ZERO.signum(), $vec2::ONE);
             assert_eq!(-$vec2::ZERO.signum(), -$vec2::ONE);
             assert_eq!($vec2::ONE.signum(), $vec2::ONE);
@@ -540,17 +516,15 @@ macro_rules! impl_vec2_float_tests {
             assert_eq!($vec2::splat(INFINITY).signum(), $vec2::ONE);
             assert_eq!($vec2::splat(NEG_INFINITY).signum(), -$vec2::ONE);
             assert!($vec2::splat(NAN).signum().is_nan_mask().all());
-        }
+        });
 
-        #[test]
-        fn test_abs() {
+        glam_test!(test_abs, {
             assert_eq!($vec2::ZERO.abs(), $vec2::ZERO);
             assert_eq!($vec2::ONE.abs(), $vec2::ONE);
             assert_eq!((-$vec2::ONE).abs(), $vec2::ONE);
-        }
+        });
 
-        #[test]
-        fn test_round() {
+        glam_test!(test_round, {
             assert_eq!($vec2::new(1.35, 0.0).round().x, 1.0);
             assert_eq!($vec2::new(0.0, 1.5).round().y, 2.0);
             assert_eq!($vec2::new(0.0, -15.5).round().y, -16.0);
@@ -563,10 +537,9 @@ macro_rules! impl_vec2_float_tests {
                 $vec2::new(NEG_INFINITY, INFINITY)
             );
             assert!($vec2::new(NAN, 0.0).round().x.is_nan());
-        }
+        });
 
-        #[test]
-        fn test_floor() {
+        glam_test!(test_floor, {
             assert_eq!($vec2::new(1.35, -1.5).floor(), $vec2::new(1.0, -2.0));
             assert_eq!(
                 $vec2::new(INFINITY, NEG_INFINITY).floor(),
@@ -577,20 +550,18 @@ macro_rules! impl_vec2_float_tests {
                 $vec2::new(-2000000.123, 10000000.123).floor(),
                 $vec2::new(-2000001.0, 10000000.0)
             );
-        }
+        });
 
-        #[test]
-        fn test_fract() {
+        glam_test!(test_fract, {
             assert_approx_eq!($vec2::new(1.35, -1.5).fract(), $vec2::new(0.35, 0.5));
             assert_approx_eq!(
                 $vec2::new(-2000000.123, 1000000.123).fract(),
                 $vec2::new(0.877, 0.123),
                 0.002
             );
-        }
+        });
 
-        #[test]
-        fn test_ceil() {
+        glam_test!(test_ceil, {
             assert_eq!($vec2::new(1.35, -1.5).ceil(), $vec2::new(2.0, -1.0));
             assert_eq!(
                 $vec2::new(INFINITY, NEG_INFINITY).ceil(),
@@ -601,42 +572,37 @@ macro_rules! impl_vec2_float_tests {
                 $vec2::new(-2000000.123, 1000000.123).ceil(),
                 $vec2::new(-2000000.0, 1000001.0)
             );
-        }
+        });
 
-        #[test]
-        fn test_lerp() {
+        glam_test!(test_lerp, {
             let v0 = $vec2::new(-1.0, -1.0);
             let v1 = $vec2::new(1.0, 1.0);
             assert_approx_eq!(v0, v0.lerp(v1, 0.0));
             assert_approx_eq!(v1, v0.lerp(v1, 1.0));
             assert_approx_eq!($vec2::ZERO, v0.lerp(v1, 0.5));
-        }
+        });
 
-        #[test]
-        fn test_is_finite() {
+        glam_test!(test_is_finite, {
             assert!($vec2::new(0.0, 0.0).is_finite());
             assert!($vec2::new(-1e-10, 1e10).is_finite());
             assert!(!$vec2::new(INFINITY, 0.0).is_finite());
             assert!(!$vec2::new(0.0, NAN).is_finite());
             assert!(!$vec2::new(0.0, NEG_INFINITY).is_finite());
             assert!(!$vec2::new(INFINITY, NEG_INFINITY).is_finite());
-        }
+        });
 
-        #[test]
-        fn test_powf() {
+        glam_test!(test_powf, {
             assert_eq!($vec2::new(2.0, 4.0).powf(2.0), $vec2::new(4.0, 16.0));
-        }
+        });
 
-        #[test]
-        fn test_exp() {
+        glam_test!(test_exp, {
             assert_eq!(
                 $vec2::new(1.0, 2.0).exp(),
                 $vec2::new((1.0 as $t).exp(), (2.0 as $t).exp())
             );
-        }
+        });
 
-        #[test]
-        fn test_angle_between() {
+        glam_test!(test_angle_between, {
             let angle = $vec2::new(1.0, 0.0).angle_between($vec2::new(0.0, 1.0));
             assert_approx_eq!(core::$t::consts::FRAC_PI_2, angle, 1e-6);
 
@@ -645,10 +611,9 @@ macro_rules! impl_vec2_float_tests {
 
             let angle = $vec2::new(-1.0, 0.0).angle_between($vec2::new(0.0, 1.0));
             assert_approx_eq!(-core::$t::consts::FRAC_PI_2, angle, 1e-6);
-        }
+        });
 
-        #[test]
-        fn test_clamp_length() {
+        glam_test!(test_clamp_length, {
             // Too long gets shortened
             assert_eq!(
                 $vec2::new(12.0, 16.0).clamp_length(7.0, 10.0),
@@ -665,10 +630,9 @@ macro_rules! impl_vec2_float_tests {
                 $vec2::new(6.0, 8.0) // lengthened to length 10.0
             );
             should_glam_assert!({ $vec2::ONE.clamp_length(1.0, 0.0) });
-        }
+        });
 
-        #[test]
-        fn test_clamp_length_max() {
+        glam_test!(test_clamp_length_max, {
             // Too long gets shortened
             assert_eq!(
                 $vec2::new(12.0, 16.0).clamp_length_max(10.0),
@@ -679,10 +643,9 @@ macro_rules! impl_vec2_float_tests {
                 $vec2::new(2.0, 1.0).clamp_length_max(5.0),
                 $vec2::new(2.0, 1.0) // unchanged
             );
-        }
+        });
 
-        #[test]
-        fn test_clamp_length_min() {
+        glam_test!(test_clamp_length_min, {
             // Not too short is unchanged
             assert_eq!(
                 $vec2::new(2.0, 1.0).clamp_length_min(0.5),
@@ -693,49 +656,174 @@ macro_rules! impl_vec2_float_tests {
                 $vec2::new(0.6, 0.8).clamp_length_min(10.0),
                 $vec2::new(6.0, 8.0) // lengthened to length 10.0
             );
-        }
+        });
 
         #[cfg(any(feature = "glam-assert", feature = "debug-glam-assert"))]
-        #[test]
-        fn test_float_glam_assert() {
+        glam_test!(test_float_glam_assert, {
             use std::panic::catch_unwind;
 
             assert!(catch_unwind(|| $vec2::ZERO.normalize()).is_err());
+        });
+    };
+}
+
+macro_rules! impl_vec2_scalar_shift_op_test {
+    ($vec2:ident, $t_min:literal, $t_max:literal, $rhs_min:literal, $rhs_max:literal) => {
+        glam_test!(test_vec2_scalar_shift_ops, {
+            for x in $t_min..$t_max {
+                for y in $t_min..$t_max {
+                    for rhs in $rhs_min..$rhs_max {
+                        assert_eq!($vec2::new(x, y) << rhs, $vec2::new(x << rhs, y << rhs));
+                        assert_eq!($vec2::new(x, y) >> rhs, $vec2::new(x >> rhs, y >> rhs));
+                    }
+                }
+            }
+        });
+    };
+}
+
+macro_rules! impl_vec2_scalar_shift_op_tests {
+    ($vec2:ident, $t_min:literal, $t_max:literal) => {
+        mod shift_by_i8 {
+            use glam::$vec2;
+            impl_vec2_scalar_shift_op_test!($vec2, $t_min, $t_max, 0i8, 2);
         }
+        mod shift_by_i16 {
+            use glam::$vec2;
+            impl_vec2_scalar_shift_op_test!($vec2, $t_min, $t_max, 0i16, 2);
+        }
+        mod shift_by_i32 {
+            use glam::$vec2;
+            impl_vec2_scalar_shift_op_test!($vec2, $t_min, $t_max, 0i32, 2);
+        }
+        mod shift_by_u8 {
+            use glam::$vec2;
+            impl_vec2_scalar_shift_op_test!($vec2, $t_min, $t_max, 0u8, 2);
+        }
+        mod shift_by_u16 {
+            use glam::$vec2;
+            impl_vec2_scalar_shift_op_test!($vec2, $t_min, $t_max, 0u16, 2);
+        }
+        mod shift_by_u32 {
+            use glam::$vec2;
+            impl_vec2_scalar_shift_op_test!($vec2, $t_min, $t_max, 0u32, 2);
+        }
+    };
+}
+
+macro_rules! impl_vec2_shift_op_test {
+    ($vec2:ident, $rhs:ident, $t_min:literal, $t_max:literal) => {
+        glam_test!(test_vec2_shift_ops, {
+            for x1 in $t_min..$t_max {
+                for y1 in $t_min..$t_max {
+                    for x2 in $t_min..$t_max {
+                        for y2 in $t_min..$t_max {
+                            assert_eq!(
+                                $vec2::new(x1, y1) << $rhs::new(x2, y2),
+                                $vec2::new(x1 << x2, y1 << y2)
+                            );
+                            assert_eq!(
+                                $vec2::new(x1, y1) >> $rhs::new(x2, y2),
+                                $vec2::new(x1 >> x2, y1 >> y2)
+                            );
+                        }
+                    }
+                }
+            }
+        });
+    };
+}
+
+macro_rules! impl_vec2_shift_op_tests {
+    ($vec2:ident) => {
+        mod shift_ivec2_by_ivec2 {
+            use super::*;
+            impl_vec2_shift_op_test!($vec2, IVec2, 0, 2);
+        }
+        mod shift_ivec2_by_uvec2 {
+            use super::*;
+            impl_vec2_shift_op_test!($vec2, UVec2, 0, 2);
+        }
+    };
+}
+
+macro_rules! impl_vec2_scalar_bit_op_tests {
+    ($vec2:ident, $t_min:literal, $t_max:literal) => {
+        glam_test!(test_vec2_scalar_bit_ops, {
+            for x in $t_min..$t_max {
+                for y in $t_min..$t_max {
+                    for rhs in $t_min..$t_max {
+                        assert_eq!($vec2::new(x, y) & rhs, $vec2::new(x & rhs, y & rhs));
+                        assert_eq!($vec2::new(x, y) | rhs, $vec2::new(x | rhs, y | rhs));
+                        assert_eq!($vec2::new(x, y) ^ rhs, $vec2::new(x ^ rhs, y ^ rhs));
+                    }
+                }
+            }
+        });
+    };
+}
+
+macro_rules! impl_vec2_bit_op_tests {
+    ($vec2:ident, $t_min:literal, $t_max:literal) => {
+        glam_test!(test_vec2_bit_ops, {
+            for x1 in $t_min..$t_max {
+                for y1 in $t_min..$t_max {
+                    assert_eq!(!$vec2::new(x1, y1), $vec2::new(!x1, !y1));
+
+                    for x2 in $t_min..$t_max {
+                        for y2 in $t_min..$t_max {
+                            assert_eq!(
+                                $vec2::new(x1, y1) & $vec2::new(x2, y2),
+                                $vec2::new(x1 & x2, y1 & y2)
+                            );
+                            assert_eq!(
+                                $vec2::new(x1, y1) | $vec2::new(x2, y2),
+                                $vec2::new(x1 | x2, y1 | y2)
+                            );
+                            assert_eq!(
+                                $vec2::new(x1, y1) ^ $vec2::new(x2, y2),
+                                $vec2::new(x1 ^ x2, y1 ^ y2)
+                            );
+                        }
+                    }
+                }
+            }
+        });
     };
 }
 
 mod vec2 {
     use glam::{const_vec2, vec2, BVec2, Mat2, Vec2, Vec3};
 
-    #[test]
-    fn test_align() {
+    glam_test!(test_align, {
         use core::mem;
         assert_eq!(8, mem::size_of::<Vec2>());
+        #[cfg(not(feature = "cuda"))]
         assert_eq!(4, mem::align_of::<Vec2>());
+        #[cfg(feature = "cuda")]
+        assert_eq!(8, mem::align_of::<Vec2>());
         assert_eq!(2, mem::size_of::<BVec2>());
         assert_eq!(1, mem::align_of::<BVec2>());
-    }
+    });
 
-    #[test]
-    fn test_as() {
+    glam_test!(test_as, {
         use glam::{DVec2, IVec2, UVec2};
-        assert_eq!(DVec2::new(-1.0, -2.0), Vec2::new(-1.0, -2.0).as_f64());
-        assert_eq!(IVec2::new(-1, -2), Vec2::new(-1.0, -2.0).as_i32());
-        assert_eq!(UVec2::new(1, 2), Vec2::new(1.0, 2.0).as_u32());
+        assert_eq!(DVec2::new(-1.0, -2.0), Vec2::new(-1.0, -2.0).as_dvec2());
+        assert_eq!(IVec2::new(-1, -2), Vec2::new(-1.0, -2.0).as_ivec2());
+        assert_eq!(UVec2::new(1, 2), Vec2::new(1.0, 2.0).as_uvec2());
 
-        assert_eq!(IVec2::new(-1, -2), DVec2::new(-1.0, -2.0).as_i32());
-        assert_eq!(UVec2::new(1, 2), DVec2::new(1.0, 2.0).as_u32());
-        assert_eq!(Vec2::new(-1.0, -2.0), DVec2::new(-1.0, -2.0).as_f32());
+        assert_eq!(IVec2::new(-1, -2), DVec2::new(-1.0, -2.0).as_ivec2());
+        assert_eq!(UVec2::new(1, 2), DVec2::new(1.0, 2.0).as_uvec2());
+        assert_eq!(Vec2::new(-1.0, -2.0), DVec2::new(-1.0, -2.0).as_vec2());
 
-        assert_eq!(DVec2::new(-1.0, -2.0), IVec2::new(-1, -2).as_f64());
-        assert_eq!(UVec2::new(1, 2), IVec2::new(1, 2).as_u32());
-        assert_eq!(Vec2::new(-1.0, -2.0), IVec2::new(-1, -2).as_f32());
+        assert_eq!(DVec2::new(-1.0, -2.0), IVec2::new(-1, -2).as_dvec2());
+        assert_eq!(UVec2::new(1, 2), IVec2::new(1, 2).as_uvec2());
+        assert_eq!(Vec2::new(-1.0, -2.0), IVec2::new(-1, -2).as_vec2());
 
-        assert_eq!(DVec2::new(1.0, 2.0), UVec2::new(1, 2).as_f64());
-        assert_eq!(IVec2::new(1, 2), UVec2::new(1, 2).as_i32());
-        assert_eq!(Vec2::new(1.0, 2.0), UVec2::new(1, 2).as_f32());
-    }
+        assert_eq!(DVec2::new(1.0, 2.0), UVec2::new(1, 2).as_dvec2());
+        assert_eq!(IVec2::new(1, 2), UVec2::new(1, 2).as_ivec2());
+        assert_eq!(Vec2::new(1.0, 2.0), UVec2::new(1, 2).as_vec2());
+    });
 
     impl_vec2_float_tests!(f32, const_vec2, vec2, Vec2, Vec3, BVec2, Mat2);
 }
@@ -743,46 +831,64 @@ mod vec2 {
 mod dvec2 {
     use glam::{const_dvec2, dvec2, BVec2, DMat2, DVec2, DVec3};
 
-    #[test]
-    fn test_align() {
+    glam_test!(test_align, {
         use core::mem;
         assert_eq!(16, mem::size_of::<DVec2>());
+        #[cfg(not(feature = "cuda"))]
         assert_eq!(mem::align_of::<f64>(), mem::align_of::<DVec2>());
+        #[cfg(feature = "cuda")]
+        assert_eq!(16, mem::align_of::<DVec2>());
         assert_eq!(2, mem::size_of::<BVec2>());
         assert_eq!(1, mem::align_of::<BVec2>());
-    }
+    });
 
     impl_vec2_float_tests!(f64, const_dvec2, dvec2, DVec2, DVec3, BVec2, DMat2);
 }
 
 mod ivec2 {
-    use glam::{const_ivec2, ivec2, BVec2, IVec2, IVec3};
+    use glam::{const_ivec2, ivec2, BVec2, IVec2, IVec3, UVec2};
 
-    #[test]
-    fn test_align() {
+    glam_test!(test_align, {
         use core::mem;
         assert_eq!(8, mem::size_of::<IVec2>());
+        #[cfg(not(feature = "cuda"))]
         assert_eq!(4, mem::align_of::<IVec2>());
+        #[cfg(feature = "cuda")]
+        assert_eq!(8, mem::align_of::<IVec2>());
         assert_eq!(2, mem::size_of::<BVec2>());
         assert_eq!(1, mem::align_of::<BVec2>());
-    }
+    });
 
     impl_vec2_signed_tests!(i32, const_ivec2, ivec2, IVec2, IVec3, BVec2);
     impl_vec2_eq_hash_tests!(i32, ivec2);
+
+    impl_vec2_scalar_shift_op_tests!(IVec2, -2, 2);
+    impl_vec2_shift_op_tests!(IVec2);
+
+    impl_vec2_scalar_bit_op_tests!(IVec2, -2, 2);
+    impl_vec2_bit_op_tests!(IVec2, -2, 2);
 }
 
 mod uvec2 {
-    use glam::{const_uvec2, uvec2, BVec2, UVec2, UVec3};
+    use glam::{const_uvec2, uvec2, BVec2, IVec2, UVec2, UVec3};
 
-    #[test]
-    fn test_align() {
+    glam_test!(test_align, {
         use core::mem;
         assert_eq!(8, mem::size_of::<UVec2>());
+        #[cfg(not(feature = "cuda"))]
         assert_eq!(4, mem::align_of::<UVec2>());
+        #[cfg(feature = "cuda")]
+        assert_eq!(8, mem::align_of::<UVec2>());
         assert_eq!(2, mem::size_of::<BVec2>());
         assert_eq!(1, mem::align_of::<BVec2>());
-    }
+    });
 
     impl_vec2_tests!(u32, const_uvec2, uvec2, UVec2, UVec3, BVec2);
     impl_vec2_eq_hash_tests!(u32, uvec2);
+
+    impl_vec2_scalar_shift_op_tests!(UVec2, 0, 2);
+    impl_vec2_shift_op_tests!(UVec2);
+
+    impl_vec2_scalar_bit_op_tests!(UVec2, 0, 2);
+    impl_vec2_bit_op_tests!(UVec2, 0, 2);
 }
