@@ -3,24 +3,21 @@ mod support;
 
 macro_rules! impl_vec4_tests {
     ($t:ident, $const_new:ident, $new:ident, $vec4:ident, $vec3:ident, $vec2:ident, $mask:ident) => {
-        #[test]
-        fn test_const() {
+        glam_test!(test_const, {
             const V: $vec4 = $const_new!([1 as $t, 2 as $t, 3 as $t, 4 as $t]);
             assert_eq!($vec4::new(1 as $t, 2 as $t, 3 as $t, 4 as $t), V);
-        }
+        });
 
-        #[test]
-        fn test_vec4_consts() {
+        glam_test!(test_vec4_consts, {
             assert_eq!($vec4::ZERO, $new(0 as $t, 0 as $t, 0 as $t, 0 as $t));
             assert_eq!($vec4::ONE, $new(1 as $t, 1 as $t, 1 as $t, 1 as $t));
             assert_eq!($vec4::X, $new(1 as $t, 0 as $t, 0 as $t, 0 as $t));
             assert_eq!($vec4::Y, $new(0 as $t, 1 as $t, 0 as $t, 0 as $t));
             assert_eq!($vec4::Z, $new(0 as $t, 0 as $t, 1 as $t, 0 as $t));
             assert_eq!($vec4::W, $new(0 as $t, 0 as $t, 0 as $t, 1 as $t));
-        }
+        });
 
-        #[test]
-        fn test_new() {
+        glam_test!(test_new, {
             let v = $new(1 as $t, 2 as $t, 3 as $t, 4 as $t);
 
             assert_eq!(v.x, 1 as $t);
@@ -50,6 +47,11 @@ macro_rules! impl_vec4_tests {
                 $vec4::from(($vec3::new(1 as $t, 2 as $t, 3 as $t), 4 as $t))
             );
 
+            assert_eq!(
+                v,
+                $vec4::from((1 as $t, $vec3::new(2 as $t, 3 as $t, 4 as $t)))
+            );
+
             assert_eq!(
                 v,
                 $vec4::from(($vec2::new(1 as $t, 2 as $t), 3 as $t, 4 as $t))
@@ -58,10 +60,9 @@ macro_rules! impl_vec4_tests {
                 v,
                 $vec4::from(($vec2::new(1 as $t, 2 as $t), $vec2::new(3 as $t, 4 as $t)))
             );
-        }
+        });
 
-        #[test]
-        fn test_fmt() {
+        glam_test!(test_fmt, {
             let a = $vec4::new(1 as $t, 2 as $t, 3 as $t, 4 as $t);
             assert_eq!(
                 format!("{:?}", a),
@@ -79,23 +80,20 @@ macro_rules! impl_vec4_tests {
             //     "$vec4(\n    1.0,\n    2.0,\n    3.0,\n    4.0\n)"
             // );
             assert_eq!(format!("{}", a), "[1, 2, 3, 4]");
-        }
+        });
 
-        #[test]
-        fn test_zero() {
+        glam_test!(test_zero, {
             let v = $vec4::ZERO;
             assert_eq!((0 as $t, 0 as $t, 0 as $t, 0 as $t), v.into());
             assert_eq!(v, $vec4::default());
-        }
+        });
 
-        #[test]
-        fn test_splat() {
+        glam_test!(test_splat, {
             let v = $vec4::splat(1 as $t);
             assert_eq!($vec4::ONE, v);
-        }
+        });
 
-        #[test]
-        fn test_accessors() {
+        glam_test!(test_accessors, {
             let mut a = $vec4::ZERO;
             a.x = 1 as $t;
             a.y = 2 as $t;
@@ -117,10 +115,9 @@ macro_rules! impl_vec4_tests {
             assert_eq!(3 as $t, a[2]);
             assert_eq!(4 as $t, a[3]);
             assert_eq!((1 as $t, 2 as $t, 3 as $t, 4 as $t), a.into());
-        }
+        });
 
-        #[test]
-        fn test_dot_unsigned() {
+        glam_test!(test_dot_unsigned, {
             let x = $new(1 as $t, 0 as $t, 0 as $t, 0 as $t);
             let y = $new(0 as $t, 1 as $t, 0 as $t, 0 as $t);
             let z = $new(0 as $t, 0 as $t, 1 as $t, 0 as $t);
@@ -129,51 +126,57 @@ macro_rules! impl_vec4_tests {
             assert_eq!(0 as $t, x.dot(y));
             assert_eq!(0 as $t, y.dot(z));
             assert_eq!(0 as $t, z.dot(w));
-        }
+        });
 
-        #[test]
-        fn test_ops() {
+        glam_test!(test_ops, {
             let a = $new(2 as $t, 4 as $t, 8 as $t, 16 as $t);
-            assert_eq!((4 as $t, 8 as $t, 16 as $t, 32 as $t), (a + a).into());
-            assert_eq!((0 as $t, 0 as $t, 0 as $t, 0 as $t), (a - a).into());
-            assert_eq!((4 as $t, 16 as $t, 64 as $t, 256 as $t), (a * a).into());
-            assert_eq!((4 as $t, 8 as $t, 16 as $t, 32 as $t), (a * 2 as $t).into());
-            assert_eq!((4 as $t, 8 as $t, 16 as $t, 32 as $t), (2 as $t * a).into());
-            assert_eq!((1 as $t, 1 as $t, 1 as $t, 1 as $t), (a / a).into());
-            assert_eq!((1 as $t, 2 as $t, 4 as $t, 8 as $t), (a / 2 as $t).into());
-            assert_eq!((8 as $t, 4 as $t, 2 as $t, 1 as $t), (16 as $t / a).into());
-        }
+            assert_eq!($new(4 as $t, 8 as $t, 16 as $t, 32 as $t), a + a);
+            assert_eq!($new(0 as $t, 0 as $t, 0 as $t, 0 as $t), a - a);
+            assert_eq!($new(4 as $t, 16 as $t, 64 as $t, 256 as $t), a * a);
+            assert_eq!($new(4 as $t, 8 as $t, 16 as $t, 32 as $t), a * 2 as $t);
+            assert_eq!($new(4 as $t, 8 as $t, 16 as $t, 32 as $t), 2 as $t * a);
+            assert_eq!($new(1 as $t, 1 as $t, 1 as $t, 1 as $t), a / a);
+            assert_eq!($new(1 as $t, 2 as $t, 4 as $t, 8 as $t), a / 2 as $t);
+            assert_eq!($new(8 as $t, 4 as $t, 2 as $t, 1 as $t), 16 as $t / a);
+            assert_eq!($new(0 as $t, 0 as $t, 0 as $t, 0 as $t), a % a);
+            assert_eq!($new(0 as $t, 1 as $t, 1 as $t, 1 as $t), a % (a - 1 as $t));
+            assert_eq!($new(0 as $t, 0 as $t, 0 as $t, 0 as $t), a % 1 as $t);
+            assert_eq!($new(2 as $t, 1 as $t, 2 as $t, 1 as $t), a % 3 as $t);
+            assert_eq!($new(1 as $t, 1 as $t, 1 as $t, 1 as $t), 17 as $t % a);
+            assert_eq!($new(2 as $t, 4 as $t, 0 as $t, 0 as $t), a % 8 as $t);
+        });
 
-        #[test]
-        fn test_assign_ops() {
+        glam_test!(test_assign_ops, {
             let a = $new(1 as $t, 2 as $t, 3 as $t, 4 as $t);
             let mut b = a;
             b += a;
-            assert_eq!((2 as $t, 4 as $t, 6 as $t, 8 as $t), b.into());
+            assert_eq!($new(2 as $t, 4 as $t, 6 as $t, 8 as $t), b);
             b -= a;
-            assert_eq!((1 as $t, 2 as $t, 3 as $t, 4 as $t), b.into());
+            assert_eq!($new(1 as $t, 2 as $t, 3 as $t, 4 as $t), b);
             b *= a;
-            assert_eq!((1 as $t, 4 as $t, 9 as $t, 16 as $t), b.into());
+            assert_eq!($new(1 as $t, 4 as $t, 9 as $t, 16 as $t), b);
             b /= a;
-            assert_eq!((1 as $t, 2 as $t, 3 as $t, 4 as $t), b.into());
+            assert_eq!($new(1 as $t, 2 as $t, 3 as $t, 4 as $t), b);
             b *= 2 as $t;
-            assert_eq!((2 as $t, 4 as $t, 6 as $t, 8 as $t), b.into());
+            assert_eq!($new(2 as $t, 4 as $t, 6 as $t, 8 as $t), b);
             b /= 2 as $t;
-            assert_eq!((1 as $t, 2 as $t, 3 as $t, 4 as $t), b.into());
-        }
+            assert_eq!($new(1 as $t, 2 as $t, 3 as $t, 4 as $t), b);
+            b %= (b + 1 as $t);
+            assert_eq!($new(1 as $t, 2 as $t, 3 as $t, 4 as $t), b);
+            b %= b;
+            assert_eq!($new(0 as $t, 0 as $t, 0 as $t, 0 as $t), b);
+        });
 
-        #[test]
-        fn test_min_max() {
+        glam_test!(test_min_max, {
             let a = $new(4 as $t, 6 as $t, 2 as $t, 8 as $t);
             let b = $new(5 as $t, 3 as $t, 7 as $t, 1 as $t);
             assert_eq!((4 as $t, 3 as $t, 2 as $t, 1 as $t), a.min(b).into());
             assert_eq!((4 as $t, 3 as $t, 2 as $t, 1 as $t), b.min(a).into());
             assert_eq!((5 as $t, 6 as $t, 7 as $t, 8 as $t), a.max(b).into());
             assert_eq!((5 as $t, 6 as $t, 7 as $t, 8 as $t), b.max(a).into());
-        }
+        });
 
-        #[test]
-        fn test_clamp() {
+        glam_test!(test_clamp, {
             fn vec(x: i32, y: i32, z: i32, w: i32) -> $vec4 {
                 $vec4::new(x as $t, y as $t, z as $t, w as $t)
             }
@@ -187,10 +190,9 @@ macro_rules! impl_vec4_tests {
             assert_eq!(vec(9, 9, 9, 9).clamp(min, max), vec(6, 6, 8, 8));
 
             should_glam_assert!({ $vec4::clamp($vec4::ZERO, $vec4::ONE, $vec4::ZERO) });
-        }
+        });
 
-        #[test]
-        fn test_hmin_hmax() {
+        glam_test!(test_hmin_hmax, {
             let a = $new(3 as $t, 4 as $t, 1 as $t, 2 as $t);
             assert_eq!(1 as $t, a.min_element());
             assert_eq!(4 as $t, a.max_element());
@@ -206,10 +208,9 @@ macro_rules! impl_vec4_tests {
                     .truncate()
                     .min_element()
             );
-        }
+        });
 
-        #[test]
-        fn test_eq() {
+        glam_test!(test_eq, {
             let a = $new(1 as $t, 1 as $t, 1 as $t, 1 as $t);
             let b = $new(1 as $t, 2 as $t, 3 as $t, 4 as $t);
             assert!(a.cmpeq(a).all());
@@ -217,10 +218,9 @@ macro_rules! impl_vec4_tests {
             assert!(a.cmpne(b).any());
             assert!(b.cmpne(a).any());
             assert!(b.cmpeq(a).any());
-        }
+        });
 
-        #[test]
-        fn test_cmp() {
+        glam_test!(test_cmp, {
             assert!(!$mask::default().any());
             assert!(!$mask::default().all());
             assert_eq!($mask::default().bitmask(), 0x0);
@@ -250,10 +250,9 @@ macro_rules! impl_vec4_tests {
             assert!(a.cmpne(b).all());
             assert!(a.cmpne(c).any());
             assert!(a == a);
-        }
+        });
 
-        #[test]
-        fn test_slice() {
+        glam_test!(test_slice, {
             let a = [1 as $t, 2 as $t, 3 as $t, 4 as $t];
             let b = $vec4::from_slice(&a);
             let c: [$t; 4] = b.into();
@@ -264,7 +263,7 @@ macro_rules! impl_vec4_tests {
 
             should_panic!({ $vec4::ONE.write_to_slice(&mut [0 as $t; 3]) });
             should_panic!({ $vec4::from_slice(&[0 as $t; 3]) });
-        }
+        });
 
         // #[test]
         // fn test_mask_as_ref() {
@@ -294,8 +293,7 @@ macro_rules! impl_vec4_tests {
         //     );
         // }
 
-        #[test]
-        fn test_mask_from() {
+        glam_test!(test_mask_from, {
             assert_eq!(
                 Into::<[u32; 4]>::into($mask::new(false, false, false, false)),
                 [0, 0, 0, 0]
@@ -320,38 +318,34 @@ macro_rules! impl_vec4_tests {
                 Into::<[u32; 4]>::into($mask::new(true, true, true, true)),
                 [!0, !0, !0, !0]
             );
-        }
+        });
 
-        #[test]
-        fn test_mask_bitmask() {
+        glam_test!(test_mask_bitmask, {
             assert_eq!($mask::new(false, false, false, false).bitmask(), 0b0000);
             assert_eq!($mask::new(false, false, true, true).bitmask(), 0b1100);
             assert_eq!($mask::new(true, true, false, false).bitmask(), 0b0011);
             assert_eq!($mask::new(false, true, false, true).bitmask(), 0b1010);
             assert_eq!($mask::new(true, false, true, false).bitmask(), 0b0101);
             assert_eq!($mask::new(true, true, true, true).bitmask(), 0b1111);
-        }
+        });
 
-        #[test]
-        fn test_mask_any() {
+        glam_test!(test_mask_any, {
             assert_eq!($mask::new(false, false, false, false).any(), false);
             assert_eq!($mask::new(true, false, false, false).any(), true);
             assert_eq!($mask::new(false, true, false, false).any(), true);
             assert_eq!($mask::new(false, false, true, false).any(), true);
             assert_eq!($mask::new(false, false, false, true).any(), true);
-        }
+        });
 
-        #[test]
-        fn test_mask_all() {
+        glam_test!(test_mask_all, {
             assert_eq!($mask::new(true, true, true, true).all(), true);
             assert_eq!($mask::new(false, true, true, true).all(), false);
             assert_eq!($mask::new(true, false, true, true).all(), false);
             assert_eq!($mask::new(true, true, false, true).all(), false);
             assert_eq!($mask::new(true, true, true, false).all(), false);
-        }
+        });
 
-        #[test]
-        fn test_mask_select() {
+        glam_test!(test_mask_select, {
             let a = $vec4::new(1 as $t, 2 as $t, 3 as $t, 4 as $t);
             let b = $vec4::new(5 as $t, 6 as $t, 7 as $t, 8 as $t);
             assert_eq!(
@@ -370,10 +364,9 @@ macro_rules! impl_vec4_tests {
                 $vec4::select($mask::new(false, false, false, false), a, b),
                 $vec4::new(5 as $t, 6 as $t, 7 as $t, 8 as $t),
             );
-        }
+        });
 
-        #[test]
-        fn test_mask_and() {
+        glam_test!(test_mask_and, {
             assert_eq!(
                 ($mask::new(false, false, false, false) & $mask::new(false, false, false, false))
                     .bitmask(),
@@ -397,10 +390,9 @@ macro_rules! impl_vec4_tests {
             let mut mask = $mask::new(true, true, false, false);
             mask &= $mask::new(true, false, true, false);
             assert_eq!(mask.bitmask(), 0b0001);
-        }
+        });
 
-        #[test]
-        fn test_mask_or() {
+        glam_test!(test_mask_or, {
             assert_eq!(
                 ($mask::new(false, false, false, false) | $mask::new(false, false, false, false))
                     .bitmask(),
@@ -424,18 +416,16 @@ macro_rules! impl_vec4_tests {
             let mut mask = $mask::new(true, true, false, false);
             mask |= $mask::new(true, false, true, false);
             assert_eq!(mask.bitmask(), 0b0111);
-        }
+        });
 
-        #[test]
-        fn test_mask_not() {
+        glam_test!(test_mask_not, {
             assert_eq!((!$mask::new(false, false, false, false)).bitmask(), 0b1111);
             assert_eq!((!$mask::new(true, true, true, true)).bitmask(), 0b0000);
             assert_eq!((!$mask::new(true, false, true, false)).bitmask(), 0b1010);
             assert_eq!((!$mask::new(false, true, false, true)).bitmask(), 0b0101);
-        }
+        });
 
-        #[test]
-        fn test_mask_fmt() {
+        glam_test!(test_mask_fmt, {
             let a = $mask::new(true, false, true, false);
 
             assert_eq!(format!("{}", a), "[true, false, true, false]");
@@ -443,10 +433,9 @@ macro_rules! impl_vec4_tests {
             //     format!("{:?}", a),
             //     format!("{}(0xffffffff, 0x0, 0xffffffff, 0x0)", stringify!($mask))
             // );
-        }
+        });
 
-        #[test]
-        fn test_mask_eq() {
+        glam_test!(test_mask_eq, {
             let a = $mask::new(true, false, true, false);
             let b = $mask::new(true, false, true, false);
             let c = $mask::new(false, true, true, false);
@@ -455,10 +444,9 @@ macro_rules! impl_vec4_tests {
             assert_eq!(b, a);
             assert_ne!(a, c);
             assert_ne!(b, c);
-        }
+        });
 
-        #[test]
-        fn test_mask_hash() {
+        glam_test!(test_mask_hash, {
             use std::collections::hash_map::DefaultHasher;
             use std::hash::Hash;
             use std::hash::Hasher;
@@ -483,29 +471,24 @@ macro_rules! impl_vec4_tests {
             assert_eq!(a_hashed, b_hashed);
             assert_ne!(a, c);
             assert_ne!(a_hashed, c_hashed);
-        }
+        });
 
-        #[test]
-        fn test_to_from_slice() {
+        glam_test!(test_to_from_slice, {
             let v = $vec4::new(1 as $t, 2 as $t, 3 as $t, 4 as $t);
             let mut a = [0 as $t, 0 as $t, 0 as $t, 0 as $t];
             v.write_to_slice(&mut a);
             assert_eq!(v, $vec4::from_slice(&a));
-        }
+        });
 
-        #[cfg(feature = "std")]
-        #[test]
-        fn test_sum() {
+        glam_test!(test_sum, {
             let one = $vec4::ONE;
             assert_eq!(vec![one, one].iter().sum::<$vec4>(), one + one);
-        }
+        });
 
-        #[cfg(feature = "std")]
-        #[test]
-        fn test_product() {
+        glam_test!(test_product, {
             let two = $vec4::new(2 as $t, 2 as $t, 2 as $t, 2 as $t);
             assert_eq!(vec![two, two].iter().product::<$vec4>(), two * two);
-        }
+        });
     };
 }
 
@@ -513,14 +496,12 @@ macro_rules! impl_vec4_signed_tests {
     ($t:ident, $const_new:ident, $new:ident, $vec4:ident, $vec3:ident, $vec2:ident, $mask:ident) => {
         impl_vec4_tests!($t, $const_new, $new, $vec4, $vec3, $vec2, $mask);
 
-        #[test]
-        fn test_neg() {
+        glam_test!(test_neg, {
             let a = $new(1 as $t, 2 as $t, 3 as $t, 4 as $t);
             assert_eq!((-1 as $t, -2 as $t, -3 as $t, -4 as $t), (-a).into());
-        }
+        });
 
-        #[test]
-        fn test_dot_signed() {
+        glam_test!(test_dot_signed, {
             let x = $new(1 as $t, 0 as $t, 0 as $t, 0 as $t);
             let y = $new(0 as $t, 1 as $t, 0 as $t, 0 as $t);
             let z = $new(0 as $t, 0 as $t, 1 as $t, 0 as $t);
@@ -529,14 +510,13 @@ macro_rules! impl_vec4_signed_tests {
             assert_eq!(0 as $t, x.dot(y));
             assert_eq!(0 as $t, x.dot(-z));
             assert_eq!(-1 as $t, w.dot(-w));
-        }
+        });
     };
 }
 
 macro_rules! impl_vec4_eq_hash_tests {
     ($t:ident, $new:ident) => {
-        #[test]
-        fn test_ve2_hash() {
+        glam_test!(test_ve2_hash, {
             use std::collections::hash_map::DefaultHasher;
             use std::hash::Hash;
             use std::hash::Hasher;
@@ -561,7 +541,7 @@ macro_rules! impl_vec4_eq_hash_tests {
             assert_eq!(a_hashed, b_hashed);
             assert_ne!(a, c);
             assert_ne!(a_hashed, c_hashed);
-        }
+        });
     };
 }
 
@@ -574,8 +554,12 @@ macro_rules! impl_vec4_float_tests {
         use core::$t::NAN;
         use core::$t::NEG_INFINITY;
 
-        #[test]
-        fn test_funcs() {
+        glam_test!(test_vec4_nan, {
+            assert!($vec4::NAN.is_nan());
+            assert!(!$vec4::NAN.is_finite());
+        });
+
+        glam_test!(test_funcs, {
             let x = $new(1.0, 0.0, 0.0, 0.0);
             let y = $new(0.0, 1.0, 0.0, 0.0);
             let z = $new(0.0, 0.0, 1.0, 0.0);
@@ -620,10 +604,9 @@ macro_rules! impl_vec4_float_tests {
                 $new(0.5, 0.25, 0.125, 0.0625),
                 $new(2.0, 4.0, 8.0, 16.0).recip()
             );
-        }
+        });
 
-        #[test]
-        fn test_project_reject() {
+        glam_test!(test_project_reject, {
             assert_eq!(
                 $new(0.0, 0.0, 0.0, 1.0),
                 $new(0.0, 1.0, 0.0, 1.0).project_onto($new(0.0, 0.0, 0.0, 2.0))
@@ -644,10 +627,9 @@ macro_rules! impl_vec4_float_tests {
             should_glam_assert!({ $vec4::ONE.reject_from($vec4::ZERO) });
             should_glam_assert!({ $vec4::ONE.project_onto_normalized($vec4::ONE) });
             should_glam_assert!({ $vec4::ONE.reject_from_normalized($vec4::ONE) });
-        }
+        });
 
-        #[test]
-        fn test_signum() {
+        glam_test!(test_signum, {
             assert_eq!($vec4::ZERO.signum(), $vec4::ONE);
             assert_eq!(-$vec4::ZERO.signum(), -$vec4::ONE);
             assert_eq!($vec4::ONE.signum(), $vec4::ONE);
@@ -655,17 +637,15 @@ macro_rules! impl_vec4_float_tests {
             assert_eq!($vec4::splat(INFINITY).signum(), $vec4::ONE);
             assert_eq!($vec4::splat(NEG_INFINITY).signum(), -$vec4::ONE);
             assert!($vec4::splat(NAN).signum().is_nan_mask().all());
-        }
+        });
 
-        #[test]
-        fn test_abs() {
+        glam_test!(test_abs, {
             assert_eq!($vec4::ZERO.abs(), $vec4::ZERO);
             assert_eq!($vec4::ONE.abs(), $vec4::ONE);
             assert_eq!((-$vec4::ONE).abs(), $vec4::ONE);
-        }
+        });
 
-        #[test]
-        fn test_round() {
+        glam_test!(test_round, {
             assert_eq!($vec4::new(1.35, 0.0, 0.0, 0.0).round().x, 1.0);
             assert_eq!($vec4::new(0.0, 1.5, 0.0, 0.0).round().y, 2.0);
             assert_eq!($vec4::new(0.0, 0.0, -15.5, 0.0).round().z, -16.0);
@@ -678,10 +658,9 @@ macro_rules! impl_vec4_float_tests {
                 $vec4::new(NEG_INFINITY, INFINITY, 1.0, -1.0)
             );
             assert!($vec4::new(NAN, 0.0, 0.0, 1.0).round().x.is_nan());
-        }
+        });
 
-        #[test]
-        fn test_floor() {
+        glam_test!(test_floor, {
             assert_eq!(
                 $vec4::new(1.35, 1.5, -1.5, 1.999).floor(),
                 $vec4::new(1.0, 1.0, -2.0, 1.0)
@@ -695,10 +674,9 @@ macro_rules! impl_vec4_float_tests {
                 $vec4::new(-0.0, -2000000.123, 10000000.123, 1000.9).floor(),
                 $vec4::new(-0.0, -2000001.0, 10000000.0, 1000.0)
             );
-        }
+        });
 
-        #[test]
-        fn test_fract() {
+        glam_test!(test_fract, {
             assert_approx_eq!(
                 $vec4::new(1.35, 1.5, -1.5, 1.999).fract(),
                 $vec4::new(0.35, 0.5, 0.5, 0.999)
@@ -708,10 +686,9 @@ macro_rules! impl_vec4_float_tests {
                 $vec4::new(0.0, 0.877, 0.123, 0.9),
                 0.002
             );
-        }
+        });
 
-        #[test]
-        fn test_ceil() {
+        glam_test!(test_ceil, {
             assert_eq!(
                 $vec4::new(1.35, 1.5, -1.5, 1234.1234).ceil(),
                 $vec4::new(2.0, 2.0, -1.0, 1235.0)
@@ -725,37 +702,33 @@ macro_rules! impl_vec4_float_tests {
                 $vec4::new(-1234.1234, -2000000.123, 1000000.123, 1000.9).ceil(),
                 $vec4::new(-1234.0, -2000000.0, 1000001.0, 1001.0)
             );
-        }
+        });
 
-        #[test]
-        fn test_lerp() {
+        glam_test!(test_lerp, {
             let v0 = $vec4::new(-1.0, -1.0, -1.0, -1.0);
             let v1 = $vec4::new(1.0, 1.0, 1.0, 1.0);
             assert_approx_eq!(v0, v0.lerp(v1, 0.0));
             assert_approx_eq!(v1, v0.lerp(v1, 1.0));
             assert_approx_eq!($vec4::ZERO, v0.lerp(v1, 0.5));
-        }
+        });
 
-        #[test]
-        fn test_is_finite() {
+        glam_test!(test_is_finite, {
             assert!($vec4::new(0.0, 0.0, 0.0, 0.0).is_finite());
             assert!($vec4::new(-1e-10, 1.0, 1e10, 42.0).is_finite());
             assert!(!$vec4::new(INFINITY, 0.0, 0.0, 0.0).is_finite());
             assert!(!$vec4::new(0.0, NAN, 0.0, 0.0).is_finite());
             assert!(!$vec4::new(0.0, 0.0, NEG_INFINITY, 0.0).is_finite());
             assert!(!$vec4::new(0.0, 0.0, 0.0, NAN).is_finite());
-        }
+        });
 
-        #[test]
-        fn test_powf() {
+        glam_test!(test_powf, {
             assert_eq!(
                 $vec4::new(2.0, 4.0, 8.0, 16.0).powf(2.0),
                 $vec4::new(4.0, 16.0, 64.0, 256.0)
             );
-        }
+        });
 
-        #[test]
-        fn test_exp() {
+        glam_test!(test_exp, {
             assert_eq!(
                 $vec4::new(1.0, 2.0, 3.0, 4.0).exp(),
                 $vec4::new(
@@ -765,10 +738,9 @@ macro_rules! impl_vec4_float_tests {
                     (4.0 as $t).exp()
                 )
             );
-        }
+        });
 
-        #[test]
-        fn test_clamp_length() {
+        glam_test!(test_clamp_length, {
             // Too long gets shortened
             assert_eq!(
                 $vec4::new(12.0, 16.0, 0.0, 0.0).clamp_length(7.0, 10.0),
@@ -785,10 +757,9 @@ macro_rules! impl_vec4_float_tests {
                 $vec4::new(6.0, 8.0, 0.0, 0.0) // lengthened to length 10.0
             );
             should_glam_assert!({ $vec4::ONE.clamp_length(1.0, 0.0) });
-        }
+        });
 
-        #[test]
-        fn test_clamp_length_max() {
+        glam_test!(test_clamp_length_max, {
             // Too long gets shortened
             assert_eq!(
                 $vec4::new(12.0, 16.0, 0.0, 0.0).clamp_length_max(10.0),
@@ -799,10 +770,9 @@ macro_rules! impl_vec4_float_tests {
                 $vec4::new(2.0, 1.0, 0.0, 0.0).clamp_length_max(5.0),
                 $vec4::new(2.0, 1.0, 0.0, 0.0) // unchanged
             );
-        }
+        });
 
-        #[test]
-        fn test_clamp_length_min() {
+        glam_test!(test_clamp_length_min, {
             // Not too short is unchanged
             assert_eq!(
                 $vec4::new(2.0, 1.0, 0.0, 0.0).clamp_length_min(0.5),
@@ -813,33 +783,211 @@ macro_rules! impl_vec4_float_tests {
                 $vec4::new(0.6, 0.8, 0.0, 0.0).clamp_length_min(10.0),
                 $vec4::new(6.0, 8.0, 0.0, 0.0) // lengthened to length 10.0
             );
+        });
+    };
+}
+macro_rules! impl_vec4_scalar_shift_op_test {
+    ($vec4:ident, $t_min:literal, $t_max:literal, $rhs_min:literal, $rhs_max:literal) => {
+        glam_test!(test_vec4_scalar_shift_ops, {
+            for x in $t_min..$t_max {
+                for y in $t_min..$t_max {
+                    for z in $t_min..$t_max {
+                        for w in $t_min..$t_max {
+                            for rhs in $rhs_min..$rhs_max {
+                                assert_eq!(
+                                    $vec4::new(x, y, z, w) << rhs,
+                                    $vec4::new(x << rhs, y << rhs, z << rhs, w << rhs)
+                                );
+                                assert_eq!(
+                                    $vec4::new(x, y, z, w) >> rhs,
+                                    $vec4::new(x >> rhs, y >> rhs, z >> rhs, w >> rhs)
+                                );
+                            }
+                        }
+                    }
+                }
+            }
+        });
+    };
+}
+
+macro_rules! impl_vec4_scalar_shift_op_tests {
+    ($vec4:ident, $t_min:literal, $t_max:literal) => {
+        mod shift_by_i8 {
+            use glam::$vec4;
+            impl_vec4_scalar_shift_op_test!($vec4, $t_min, $t_max, 0i8, 2);
+        }
+        mod shift_by_i16 {
+            use glam::$vec4;
+            impl_vec4_scalar_shift_op_test!($vec4, $t_min, $t_max, 0i16, 2);
+        }
+        mod shift_by_i32 {
+            use glam::$vec4;
+            impl_vec4_scalar_shift_op_test!($vec4, $t_min, $t_max, 0i32, 2);
+        }
+        mod shift_by_u8 {
+            use glam::$vec4;
+            impl_vec4_scalar_shift_op_test!($vec4, $t_min, $t_max, 0u8, 2);
+        }
+        mod shift_by_u16 {
+            use glam::$vec4;
+            impl_vec4_scalar_shift_op_test!($vec4, $t_min, $t_max, 0u16, 2);
+        }
+        mod shift_by_u32 {
+            use glam::$vec4;
+            impl_vec4_scalar_shift_op_test!($vec4, $t_min, $t_max, 0u32, 2);
         }
     };
 }
 
+macro_rules! impl_vec4_shift_op_test {
+    ($vec4:ident, $rhs:ident, $t_min:literal, $t_max:literal) => {
+        glam_test!(test_vec4_shift_ops, {
+            for x1 in $t_min..$t_max {
+                for y1 in $t_min..$t_max {
+                    for z1 in $t_min..$t_max {
+                        for w1 in $t_min..$t_max {
+                            for x2 in $t_min..$t_max {
+                                for y2 in $t_min..$t_max {
+                                    for z2 in $t_min..$t_max {
+                                        for w2 in $t_min..$t_max {
+                                            assert_eq!(
+                                                $vec4::new(x1, y1, z1, w1)
+                                                    << $rhs::new(x2, y2, z2, w2),
+                                                $vec4::new(x1 << x2, y1 << y2, z1 << z2, w1 << w2)
+                                            );
+                                            assert_eq!(
+                                                $vec4::new(x1, y1, z1, w1)
+                                                    >> $rhs::new(x2, y2, z2, w2),
+                                                $vec4::new(x1 >> x2, y1 >> y2, z1 >> z2, w1 >> w2)
+                                            );
+                                        }
+                                    }
+                                }
+                            }
+                        }
+                    }
+                }
+            }
+        });
+    };
+}
+
+macro_rules! impl_vec4_shift_op_tests {
+    ($vec4:ident) => {
+        mod shift_ivec4_by_ivec4 {
+            use super::*;
+            impl_vec4_shift_op_test!($vec4, IVec4, 0, 2);
+        }
+        mod shift_ivec4_by_uvec4 {
+            use super::*;
+            impl_vec4_shift_op_test!($vec4, UVec4, 0, 2);
+        }
+    };
+}
+
+macro_rules! impl_vec4_scalar_bit_op_tests {
+    ($vec4:ident, $t_min:literal, $t_max:literal) => {
+        glam_test!(test_vec4_scalar_bit_ops, {
+            for x in $t_min..$t_max {
+                for y in $t_min..$t_max {
+                    for z in $t_min..$t_max {
+                        for w in $t_min..$t_max {
+                            for rhs in $t_min..$t_max {
+                                assert_eq!(
+                                    $vec4::new(x, y, z, w) & rhs,
+                                    $vec4::new(x & rhs, y & rhs, z & rhs, w & rhs)
+                                );
+                                assert_eq!(
+                                    $vec4::new(x, y, z, w) | rhs,
+                                    $vec4::new(x | rhs, y | rhs, z | rhs, w | rhs)
+                                );
+                                assert_eq!(
+                                    $vec4::new(x, y, z, w) ^ rhs,
+                                    $vec4::new(x ^ rhs, y ^ rhs, z ^ rhs, w ^ rhs)
+                                );
+                            }
+                        }
+                    }
+                }
+            }
+        });
+    };
+}
+
+macro_rules! impl_vec4_bit_op_tests {
+    ($vec4:ident, $t_min:literal, $t_max:literal) => {
+        glam_test!(test_vec4_bit_ops, {
+            for x1 in $t_min..$t_max {
+                for y1 in $t_min..$t_max {
+                    for z1 in $t_min..$t_max {
+                        for w1 in $t_min..$t_max {
+                            assert_eq!(!$vec4::new(x1, y1, z1, w1), $vec4::new(!x1, !y1, !z1, !w1));
+
+                            for x2 in $t_min..$t_max {
+                                for y2 in $t_min..$t_max {
+                                    for z2 in $t_min..$t_max {
+                                        for w2 in $t_min..$t_max {
+                                            assert_eq!(
+                                                $vec4::new(x1, y1, z1, w1)
+                                                    & $vec4::new(x2, y2, z2, w2),
+                                                $vec4::new(x1 & x2, y1 & y2, z1 & z2, w1 & w2)
+                                            );
+                                            assert_eq!(
+                                                $vec4::new(x1, y1, z1, w1)
+                                                    | $vec4::new(x2, y2, z2, w2),
+                                                $vec4::new(x1 | x2, y1 | y2, z1 | z2, w1 | w2)
+                                            );
+                                            assert_eq!(
+                                                $vec4::new(x1, y1, z1, w1)
+                                                    ^ $vec4::new(x2, y2, z2, w2),
+                                                $vec4::new(x1 ^ x2, y1 ^ y2, z1 ^ z2, w1 ^ w2)
+                                            );
+                                        }
+                                    }
+                                }
+                            }
+                        }
+                    }
+                }
+            }
+        });
+    };
+}
 mod vec4 {
     use glam::{const_vec4, vec4, Vec2, Vec3, Vec4};
 
-    #[cfg(all(target_feature = "sse2", not(feature = "scalar-math")))]
+    #[cfg(all(
+        any(target_feature = "sse2", target_feature = "simd128"),
+        not(feature = "scalar-math")
+    ))]
     type Vec4Mask = glam::BVec4A;
 
-    #[cfg(any(not(target_feature = "sse2"), feature = "scalar-math"))]
+    #[cfg(any(
+        not(any(target_feature = "sse2", target_feature = "simd128")),
+        feature = "scalar-math"
+    ))]
     type Vec4Mask = glam::BVec4;
 
-    #[test]
-    fn test_align() {
+    glam_test!(test_align, {
         use std::mem;
         assert_eq!(16, mem::size_of::<Vec4>());
-        if cfg!(any(not(target_feature = "sse2"), feature = "scalar-math")) {
-            assert_eq!(4, mem::align_of::<Vec4>());
-            assert_eq!(4, mem::size_of::<Vec4Mask>());
-            assert_eq!(1, mem::align_of::<Vec4Mask>());
-        } else {
+        if cfg!(any(not(feature = "scalar-math"), feature = "cuda")) {
             assert_eq!(16, mem::align_of::<Vec4>());
+        } else {
+            assert_eq!(4, mem::align_of::<Vec4>());
+        }
+        if cfg!(all(
+            any(target_feature = "sse2", target_feature = "simd128"),
+            not(feature = "scalar-math")
+        )) {
             assert_eq!(16, mem::size_of::<Vec4Mask>());
             assert_eq!(16, mem::align_of::<Vec4Mask>());
+        } else {
+            assert_eq!(4, mem::size_of::<Vec4Mask>());
+            assert_eq!(1, mem::align_of::<Vec4Mask>());
         }
-    }
+    });
 
     #[cfg(all(target_feature = "sse2", not(feature = "scalar-math")))]
     #[test]
@@ -874,55 +1022,66 @@ mod vec4 {
         assert_eq!([0xffffffff, 0, 0xffffffff, 0], a0.0);
     }
 
-    #[test]
-    fn test_as() {
+    glam_test!(test_as, {
         use glam::{DVec4, IVec4, UVec4};
         assert_eq!(
             DVec4::new(-1.0, -2.0, -3.0, -4.0),
-            Vec4::new(-1.0, -2.0, -3.0, -4.0).as_f64()
+            Vec4::new(-1.0, -2.0, -3.0, -4.0).as_dvec4()
         );
         assert_eq!(
             IVec4::new(-1, -2, -3, -4),
-            Vec4::new(-1.0, -2.0, -3.0, -4.0).as_i32()
+            Vec4::new(-1.0, -2.0, -3.0, -4.0).as_ivec4()
         );
         assert_eq!(
             UVec4::new(1, 2, 3, 4),
-            Vec4::new(1.0, 2.0, 3.0, 4.0).as_u32()
+            Vec4::new(1.0, 2.0, 3.0, 4.0).as_uvec4()
         );
 
         assert_eq!(
             IVec4::new(-1, -2, -3, -4),
-            DVec4::new(-1.0, -2.0, -3.0, -4.0).as_i32()
+            DVec4::new(-1.0, -2.0, -3.0, -4.0).as_ivec4()
         );
         assert_eq!(
             UVec4::new(1, 2, 3, 4),
-            DVec4::new(1.0, 2.0, 3.0, 4.0).as_u32()
+            DVec4::new(1.0, 2.0, 3.0, 4.0).as_uvec4()
         );
         assert_eq!(
             Vec4::new(-1.0, -2.0, -3.0, -4.0),
-            DVec4::new(-1.0, -2.0, -3.0, -4.0).as_f32()
+            DVec4::new(-1.0, -2.0, -3.0, -4.0).as_vec4()
         );
 
         assert_eq!(
             DVec4::new(-1.0, -2.0, -3.0, -4.0),
-            IVec4::new(-1, -2, -3, -4).as_f64()
+            IVec4::new(-1, -2, -3, -4).as_dvec4()
         );
-        assert_eq!(UVec4::new(1, 2, 3, 4), IVec4::new(1, 2, 3, 4).as_u32());
+        assert_eq!(UVec4::new(1, 2, 3, 4), IVec4::new(1, 2, 3, 4).as_uvec4());
         assert_eq!(
             Vec4::new(-1.0, -2.0, -3.0, -4.0),
-            IVec4::new(-1, -2, -3, -4).as_f32()
+            IVec4::new(-1, -2, -3, -4).as_vec4()
         );
 
         assert_eq!(
             DVec4::new(1.0, 2.0, 3.0, 4.0),
-            UVec4::new(1, 2, 3, 4).as_f64()
+            UVec4::new(1, 2, 3, 4).as_dvec4()
         );
-        assert_eq!(IVec4::new(1, 2, 3, 4), UVec4::new(1, 2, 3, 4).as_i32());
+        assert_eq!(IVec4::new(1, 2, 3, 4), UVec4::new(1, 2, 3, 4).as_ivec4());
         assert_eq!(
             Vec4::new(1.0, 2.0, 3.0, 4.0),
-            UVec4::new(1, 2, 3, 4).as_f32()
+            UVec4::new(1, 2, 3, 4).as_vec4()
         );
-    }
+    });
+
+    glam_test!(test_vec3a, {
+        use glam::Vec3A;
+        assert_eq!(
+            Vec4::new(1.0, 2.0, 3.0, 4.0),
+            Vec4::from((Vec3A::new(1.0, 2.0, 3.0), 4.0))
+        );
+        assert_eq!(
+            Vec4::new(1.0, 2.0, 3.0, 4.0),
+            Vec4::from((1.0, Vec3A::new(2.0, 3.0, 4.0)))
+        );
+    });
 
     impl_vec4_float_tests!(f32, const_vec4, vec4, Vec4, Vec3, Vec2, Vec4Mask);
 }
@@ -930,46 +1089,64 @@ mod vec4 {
 mod dvec4 {
     use glam::{const_dvec4, dvec4, BVec4, DVec2, DVec3, DVec4};
 
-    #[test]
-    fn test_align() {
+    glam_test!(test_align, {
         use std::mem;
         assert_eq!(32, mem::size_of::<DVec4>());
+        #[cfg(not(feature = "cuda"))]
         assert_eq!(mem::align_of::<f64>(), mem::align_of::<DVec4>());
+        #[cfg(feature = "cuda")]
+        assert_eq!(16, mem::align_of::<DVec4>());
         assert_eq!(4, mem::size_of::<BVec4>());
         assert_eq!(1, mem::align_of::<BVec4>());
-    }
+    });
 
     impl_vec4_float_tests!(f64, const_dvec4, dvec4, DVec4, DVec3, DVec2, BVec4);
 }
 
 mod ivec4 {
-    use glam::{const_ivec4, ivec4, BVec4, IVec2, IVec3, IVec4};
+    use glam::{const_ivec4, ivec4, BVec4, IVec2, IVec3, IVec4, UVec4};
 
-    #[test]
-    fn test_align() {
+    glam_test!(test_align, {
         use std::mem;
         assert_eq!(16, mem::size_of::<IVec4>());
+        #[cfg(not(feature = "cuda"))]
         assert_eq!(4, mem::align_of::<IVec4>());
+        #[cfg(feature = "cuda")]
+        assert_eq!(16, mem::align_of::<IVec4>());
         assert_eq!(4, mem::size_of::<BVec4>());
         assert_eq!(1, mem::align_of::<BVec4>());
-    }
+    });
 
     impl_vec4_signed_tests!(i32, const_ivec4, ivec4, IVec4, IVec3, IVec2, BVec4);
     impl_vec4_eq_hash_tests!(i32, ivec4);
+
+    impl_vec4_scalar_shift_op_tests!(IVec4, -2, 2);
+    impl_vec4_shift_op_tests!(IVec4);
+
+    impl_vec4_scalar_bit_op_tests!(IVec4, -2, 2);
+    impl_vec4_bit_op_tests!(IVec4, -2, 2);
 }
 
 mod uvec4 {
-    use glam::{const_uvec4, uvec4, BVec4, UVec2, UVec3, UVec4};
+    use glam::{const_uvec4, uvec4, BVec4, IVec4, UVec2, UVec3, UVec4};
 
-    #[test]
-    fn test_align() {
+    glam_test!(test_align, {
         use std::mem;
         assert_eq!(16, mem::size_of::<UVec4>());
+        #[cfg(not(feature = "cuda"))]
         assert_eq!(4, mem::align_of::<UVec4>());
+        #[cfg(feature = "cuda")]
+        assert_eq!(16, mem::align_of::<UVec4>());
         assert_eq!(4, mem::size_of::<BVec4>());
         assert_eq!(1, mem::align_of::<BVec4>());
-    }
+    });
 
     impl_vec4_tests!(u32, const_uvec4, uvec4, UVec4, UVec3, UVec2, BVec4);
     impl_vec4_eq_hash_tests!(u32, uvec4);
+
+    impl_vec4_scalar_shift_op_tests!(UVec4, 0, 2);
+    impl_vec4_shift_op_tests!(UVec4);
+
+    impl_vec4_scalar_bit_op_tests!(UVec4, 0, 2);
+    impl_vec4_bit_op_tests!(UVec4, 0, 2);
 }
